Unit Laservga;

(* This unit provides a procedure to hardcopy a graphic VGA screen
on an Hewlett Packard LaserJet III printer. It uses the raster graphics
capability of PCL language. For more information report to LasetJet III
Printer User's Manual. It may work with other HP printer.

Author  : Jean-didier Gayrard, CIS 76....,...
version : 0.0
date    : July 95 *)

interface
uses Dos, Printer;
(* use Printer unit to access Lst variable for convenance *)

Const  Port_Vga = $03CE;
       Esc = #27;
       FormFeed = #12;

const SameOrientation = Esc+'*r0F';
      StartGraphics = Esc+'*r1A'; (* at current cursor *)
      EndGraphics = Esc+'*rB';
      ResetPrinter = Esc + 'E';

Type Orientation = (Portrait, Landscape);
     Paper_size = (A4, Letter);
     Resolution = (Low, Medium, High);  (* 75, 150, 300 dots per inch *)

Procedure Print_VGA_Screen (Inverse : boolean; var result : boolean);
(* printscreen on HP laserjet, inverse = true then black is printed
in black, other color in white, inverse = false then black is printed
as white and other color as black *)

Implementation

(* some local procedures for more lisibility *)
Procedure Set_Orientation ( O : Orientation);
begin
case O of
   Portrait:  Write(Lst,Esc+'&l0O');
   Landscape: Write(Lst,Esc+'&l1O')
   end
end;

Procedure Set_Paper_Size ( P : Paper_size);
begin
case P of
     A4 :       Write(Lst,Esc+'&l26A');
     Letter :   Write(Lst,Esc+'&l2A')
     end
end;

Procedure Set_Resolution ( R : Resolution);
begin
case R of
     Low :      write(Lst,Esc+'*t75R');
     Medium :   write(Lst,Esc+'*t150R');
     High :     write(Lst,Esc+'*t300R')
     end
end;

Procedure Set_Cursor ( X, Y : word);
var xo, yo: string[4];
begin
str(x,xo);
str(y,yo);
write(Lst,Esc+'*p'+xo+'x'+yo+'Y')
end;

(* Useful procedure, dump VGA video memory on printer *)
Procedure Print_VGA_screen (inverse : boolean; var result : boolean);
var ModeVideo : Byte;
    X, Y : word;
    Regs : Registers;
    Offset : word;
    octet : byte;

begin
(* Video mode reading *)
Regs.AH := $0F;
Intr($10,Regs);
ModeVideo := Regs.AL;
(* Test video mode *)
if not (ModeVideo in [$10, $12])
   then result := false  (* wrong video mode *)
   else begin
       (* program VGA controler *)
       PortW[Port_Vga]:= $0805; (* set read 1 mode *)
       PortW[Port_Vga]:= $0002; (* color compare *)
       PortW[Port_Vga]:= $0F07; (* color don't care *)
       (* set up printer *)
       write(Lst,ResetPrinter);
       Set_Paper_Size(Letter);
       Set_Orientation(Landscape);
       Set_Resolution(Low);
       Set_cursor(210,0); (* to let some upside margin *)
       write(Lst,Sameorientation);
       (* Dump screen *)
       write(Lst,StartGraphics);
       for y:= 0 to 479 do
           begin
           write(Lst,Esc+'*b80W');      (* transfer 80 bytes *)
           for x:= 0 to 79 do
               begin
               offset := y * 80 + x;
               if (inverse = false) then octet := mem[$A000: offset]
                                    else octet := not(mem[$A000: offset]);
               write(Lst,chr(octet))
               end
           end;
       write(Lst,EndGraphics);
       write(Lst,FormFeed);
       result := true
       end
end;

end.


